using System;
using System.Xml;
using System.Text;

using Lab6.Model;

namespace Lab6
{
    public class Logger : IDisposable
    {
        private XmlWriter _xmlWriter;

        public Logger(string fileName)
        {
            XmlWriterSettings writerSettings = new XmlWriterSettings();
            writerSettings.Indent = true;
            _xmlWriter = XmlWriter.Create(fileName, writerSettings);
            _xmlWriter.WriteStartDocument();
            _xmlWriter.WriteStartElement("EventiBiblioteca");
        }

        public void TracciaNuovoLibro(Libro libro, bool isOk)
        {
            WriteLog("NuovoLibro", libro, isOk);
        }

        public void TracciaModificaLibro(Libro libro, bool isOk)
        {
            WriteLog("ModificaLibro", libro, isOk);
        }

        public void TracciaNuovaPersona(Persona persona, bool isOk)
        {
            WriteLog("NuovaPersona", persona, isOk);
        }

        public void TracciaModificaPersona(Persona persona, bool isOk)
        {
            WriteLog("ModificaPersona", persona, isOk);
        }

        public void TracciaNuovoPrestito(Prestito prestito, bool isOk)
        {
            WriteLog("NuovoPrestito", prestito, isOk);
        }

        public void TracciaCancellazionePrestito(Prestito prestito, bool isOk)
        {
            WriteLog("CancellazionePrestito", prestito, isOk);
        }

        public void Close()
        {
            Dispose();
        }

        private void WriteLog(string tagName, Libro libro, bool isOk)
        {
            if (libro == null)
                throw new ArgumentNullException("libro");
            _xmlWriter.WriteStartElement(tagName);
            if (!isOk)
                _xmlWriter.WriteAttributeString("operazioneRifiutata", "true");
            libro.Salva(_xmlWriter);
            _xmlWriter.WriteEndElement();
        }

        private void WriteLog(string tagName, Persona persona, bool isOk)
        {
            if (persona == null)
                throw new ArgumentNullException("persona");
            _xmlWriter.WriteStartElement(tagName);
            if (!isOk)
                _xmlWriter.WriteAttributeString("operazioneRifiutata", "true");
            persona.Salva(_xmlWriter);
            _xmlWriter.WriteEndElement();
        }

        private void WriteLog(string tagName, Prestito prestito, bool isOk)
        {
            if (prestito == null)
                throw new ArgumentNullException("prestito");
            _xmlWriter.WriteStartElement(tagName);
            if (!isOk)
                _xmlWriter.WriteAttributeString("operazioneRifiutata", "true");
            prestito.Salva(_xmlWriter);
            _xmlWriter.WriteEndElement();
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (_xmlWriter != null)
            {
                _xmlWriter.WriteEndElement();
                _xmlWriter.WriteEndDocument();
                _xmlWriter.Close();
                _xmlWriter = null;
            }
        }

        #endregion
    }
}
