﻿using System;
using System.Xml;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MiniLib.Model;

namespace Lab6.Model
{
    public partial class Biblioteca : DocumentBase
    {
        private readonly BibliotecaDataSet _dataSet = new BibliotecaDataSet();

        protected Biblioteca(string name)
            : base(name)
        {
        }

        public BibliotecaDataSet DataSet
        {
            get { return _dataSet; }
        }

        public BibliotecaDataSet.LibriDataTable LibriTable
        {
            get { return _dataSet.Libri; }
        }

        public BibliotecaDataSet.PersoneDataTable PersoneTable
        {
            get { return _dataSet.Persone; }
        }

        public BibliotecaDataSet.PrestitiDataTable PrestitiTable
        {
            get { return _dataSet.Prestiti; }
        }

        protected override void Invalidate()
        {
            base.Invalidate();
            _dataSet.AcceptChanges();
        }

        //  --- Gestione libri ---

        public Libro GetLibro(int id)
        {
            string filter = string.Format("Id = {0}", id);
            DataRow[] rows = LibriTable.Select(filter);
            return ((rows.Length == 0) ? null : new Libro(this, (BibliotecaDataSet.LibriRow) rows[0]));
        }

        public IEnumerable<Libro> GetLibri()
        {
            List<Libro> libri = new List<Libro>();
            foreach (BibliotecaDataSet.LibriRow libroRow in LibriTable.Rows)
            {
                libri.Add(new Libro(this, libroRow));
            }
            return libri;
        }

        public bool Contains(Libro libro)
        {
            if (libro == null)
                throw new ArgumentNullException("libro");
            return (GetLibro(libro.Id) != null);
        }

        public bool Aggiungi(Libro libro)
        {
            if (Contains(libro))
                return false;
            LibriTable.AddLibriRow(libro.LibroRow);
            Invalidate();
            return true;
        }

        public bool Modifica(Libro libro)
        {
            if (libro == null)
                throw new ArgumentNullException();
            //if (!Contains(libro) || libro != _libri[libro.Id])
            //    return false;
            Invalidate();
            return true;
        }

        //  --- Gestione persone ---

        public Persona GetPersona(int id)
        {
            string filter = string.Format("Id = {0}", id);
            DataRow[] rows = PersoneTable.Select(filter);
            return ((rows.Length == 0) ? null : new Persona(this, (BibliotecaDataSet.PersoneRow) rows[0]));
        }

        public IEnumerable<Persona> GetPersone()
        {
            List<Persona> persone = new List<Persona>();
            foreach (BibliotecaDataSet.PersoneRow personaRow in PersoneTable.Rows)
            {
                persone.Add(new Persona(this, personaRow));
            }
            return persone;
        }

        public bool Contains(Persona persona)
        {
            if (persona == null)
                throw new ArgumentNullException("persona");
            return (GetPersona(persona.Id) != null);
        }

        public bool Aggiungi(Persona persona)
        {
            if (Contains(persona))
                return false;
            PersoneTable.AddPersoneRow(persona.PersonaRow);
            Invalidate();
            return true;
        }

        public bool Modifica(Persona persona)
        {
            if (persona == null)
                throw new ArgumentNullException();
            //if (!Contains(persona) || persona != _persone[persona.Id])
            //    return false;
            Invalidate();
            return true;
        }

        //  --- Gestione prestiti ---

        public Prestito GetPrestito(int idLibro)
        {
            string filter = string.Format("IdLibro = {0}", idLibro);
            DataRow[] rows = PrestitiTable.Select(filter);
            return ((rows.Length == 0) ? null : new Prestito(this, (BibliotecaDataSet.PrestitiRow) rows[0]));
        }

        public IEnumerable<Prestito> GetPrestiti()
        {
            List<Prestito> prestiti = new List<Prestito>();
            foreach (BibliotecaDataSet.PrestitiRow prestitoRow in PrestitiTable.Rows)
            {
                prestiti.Add(new Prestito(this, prestitoRow));
            }
            return prestiti;
        }

        public bool IsLibroDisponibile(Libro libro)
        {
            return (GetPrestito(libro.Id) == null);
        }

        public IEnumerable<Libro> GetLibriDisponibili()
        {
            return from libroRow in LibriTable.AsEnumerable()
                   where libroRow.GetChildRows("LibriPrestiti").Length == 0
                   select new Libro(this, (BibliotecaDataSet.LibriRow) libroRow);
        }

        public bool Contains(Prestito prestito)
        {
            if (prestito == null)
                throw new ArgumentNullException("persona");
            return (GetPrestito(prestito.IdLibro) != null);
        }

        public bool Aggiungi(Prestito prestito)
        {
            if (Contains(prestito))
                return false;
            PrestitiTable.AddPrestitiRow(prestito.PrestitoRow);
            Invalidate();
            return true;
        }

        public bool Rimuovi(Prestito prestito)
        {
            if (!Contains(prestito))
                return false;
            prestito.PrestitoRow.Delete();
            Invalidate();
            return true;
        }

        //  --- Supporto alla persistenza ---

        public void Load(XmlReader reader)
        {
            DataSet.ReadXml(reader);
            DataSet.AcceptChanges();
        }

        public void Salva(XmlWriter writer)
        {
            DataSet.WriteXml(writer, XmlWriteMode.IgnoreSchema);
        }
    }
}
