using System;
using System.Xml;

using MiniLib.Model;

namespace Lab6.Model
{
    public class Libro
    {
        private readonly Biblioteca _owner;
        private readonly BibliotecaDataSet.LibriRow _libroRow;

        public Libro(Biblioteca owner)
            : this(owner, String.Empty, String.Empty, 0)
        {
        }

        public Libro(Biblioteca owner, string titolo, string autore, int anno)
        {
            if (owner == null)
                throw new ArgumentNullException("owner");
            if (titolo == null)
                throw new ArgumentNullException("titolo");
            if (autore == null)
                throw new ArgumentNullException("autore");
            _owner = owner;
            _libroRow = _owner.LibriTable.NewLibriRow();
            Titolo = titolo;
            Autore = autore;
            Anno = anno;
        }

        public Libro(Biblioteca owner, BibliotecaDataSet.LibriRow libroRow)
        {
            if (owner == null)
                throw new ArgumentNullException("owner");
            if (libroRow == null)
                throw new ArgumentNullException("libroRow");
            _owner = owner;
            _libroRow = libroRow;
        }

        public Biblioteca Owner
        {
            get { return _owner; }
        }

        public BibliotecaDataSet.LibriRow LibroRow
        {
            get { return _libroRow; }
        }

        [Editable("Identificatore", 30)]
        public int Id
        {
            get { return _libroRow.Id; }
            //private set { _libroRow.Id = value; }
        }

        [Editable("Titolo", 100)]
        public string Titolo
        {
            get { return _libroRow.Titolo; }
            set { _libroRow.Titolo = value; }
        }

        [Editable("Autore", 100)]
        public string Autore
        {
            get { return _libroRow.Autore; }
            set { _libroRow.Autore = value; }
        }

        [Editable("Anno", 50)]
        public int Anno
        {
            get { return _libroRow.Anno; }
            set { _libroRow.Anno = value; }
        }

        [Editable("Disponibile", 40)]
        public bool Disponibile
        {
            get { return _libroRow.Disponibile; }
        }

        public void Salva(XmlWriter writer)
        {
            writer.WriteStartElement("Libro");
            writer.WriteAttributeString("id", XmlConvert.ToString(Id));
            writer.WriteAttributeString("titolo", Titolo);
            writer.WriteAttributeString("autore", Autore);
            writer.WriteAttributeString("anno", XmlConvert.ToString(Anno));
            writer.WriteEndElement();
        }

        public override string ToString()
        {
            return String.Format("{{{0}, {1}}}", Id, Titolo);
        }
    }
}