using System;
using System.Xml;

using MiniLib.Model;

namespace Lab6.Model
{
    public class Persona
    {
        private readonly Biblioteca _owner;
        private readonly BibliotecaDataSet.PersoneRow _personaRow;

        public Persona(Biblioteca owner)
            : this(owner, String.Empty, String.Empty, String.Empty)
        {
        }

        public Persona(Biblioteca owner, string nome, string indirizzo, string telefono)
        {
            if (owner == null)
                throw new ArgumentNullException("owner");
            if (nome == null)
                throw new ArgumentNullException("nome");
            if (indirizzo == null)
                throw new ArgumentNullException("indirizzo");
            if (telefono == null)
                throw new ArgumentNullException("telefono");
            _owner = owner;
            _personaRow = _owner.PersoneTable.NewPersoneRow();
            Nome = nome;
            Indirizzo = indirizzo;
            Telefono = telefono;
        }

        public Persona(Biblioteca owner, BibliotecaDataSet.PersoneRow personaRow)
        {
            if (owner == null)
                throw new ArgumentNullException("owner");
            if (personaRow == null)
                throw new ArgumentNullException("personaRow");
            _owner = owner;
            _personaRow = personaRow;
        }

        public Biblioteca Owner
        {
            get { return _owner; }
        }

        public BibliotecaDataSet.PersoneRow PersonaRow
        {
            get { return _personaRow; }
        }

        [Editable("Identificatore", 30)]
        public int Id
        {
            get { return _personaRow.Id; }
            //private set { _personaRow.Id = value; }
        }

        [Editable("Nome", 100)]
        public string Nome
        {
            get { return _personaRow.Nome; }
            set { _personaRow.Nome = value; }
        }

        [Editable("Indirizzo", 100)]
        public string Indirizzo
        {
            get { return _personaRow.Indirizzo; }
            set { _personaRow.Indirizzo = value; }
        }

        [Editable("Numero di Telefono", 80)]
        public string Telefono
        {
            get { return _personaRow.Telefono; }
            set { _personaRow.Telefono = value; }
        }

        public void Salva(XmlWriter writer)
        {
            writer.WriteStartElement("Persona");
            writer.WriteAttributeString("id", XmlConvert.ToString(Id));
            writer.WriteAttributeString("nome", Nome);
            writer.WriteAttributeString("indirizzo", Indirizzo);
            writer.WriteAttributeString("telefono", Telefono);
            writer.WriteEndElement();
        }

        public override string ToString()
        {
            return String.Format("{{{0}, {1}}}", Id, Nome);
        }
    }
}
