using System;
using System.Xml;

namespace Lab6.Model
{
    public class Prestito
    {
        private readonly Biblioteca _owner;
        private readonly BibliotecaDataSet.PrestitiRow _prestitoRow;

        public Prestito(Biblioteca owner, Libro libro, Persona persona)
        {
            if (owner == null)
                throw new ArgumentNullException("owner");
            if (libro == null)
                throw new ArgumentNullException("libro");
            if (persona == null)
                throw new ArgumentNullException("persona");
            _owner = owner;
            _prestitoRow = _owner.PrestitiTable.NewPrestitiRow();
            IdLibro = libro.Id;
            IdPersona = persona.Id;
        }

        public Prestito(Biblioteca owner, BibliotecaDataSet.PrestitiRow prestitoRow)
        {
            if (owner == null)
                throw new ArgumentNullException("owner");
            if (prestitoRow == null)
                throw new ArgumentNullException("prestitoRow");
            _owner = owner;
            _prestitoRow = prestitoRow;
        }

        public Biblioteca Owner
        {
            get { return _owner; }
        }

        public BibliotecaDataSet.PrestitiRow PrestitoRow
        {
            get { return _prestitoRow; }
        }

        public int IdLibro
        {
            get { return _prestitoRow.IdLibro; }
            private set { _prestitoRow.IdLibro = value; }
        }

        public int IdPersona
        {
            get { return _prestitoRow.IdPersona; }
            private set { _prestitoRow.IdPersona = value; }
        }

        public Libro Libro
        {
            get { return _owner.GetLibro(IdLibro); }
        }

        public Persona Persona
        {
            get { return _owner.GetPersona(IdPersona); }
        }

        public string TitoloLibro
        {
            get { return _prestitoRow.Libro; }
        }

        public string NomePersona
        {
            get { return _prestitoRow.Persona; }
        }

        public void Salva(XmlWriter writer)
        {
            writer.WriteStartElement("Prestito");
            writer.WriteAttributeString("idLibro", XmlConvert.ToString(IdLibro));
            writer.WriteAttributeString("idPersona", XmlConvert.ToString(IdPersona));
            writer.WriteEndElement();
        }

        public override string ToString()
        {
            return Libro.ToString() + " - " + Persona.ToString();
        }
    }
}
