﻿using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Lab6.Model;
using MiniLib.Model;
using MiniLib.Presentation;

namespace Lab6.Presentation
{
    public class BibliotecaController : ControllerBase
    {
        public BibliotecaController(BibliotecaForm bibliotecaForm, Biblioteca.BibliotecaManager bibliotecaManager)
            : base(bibliotecaForm, bibliotecaManager)
        {
        }

        private BibliotecaForm BibliotecaForm
        {
            get { return (BibliotecaForm) ViewContainer; }
        }

        public Biblioteca.BibliotecaManager BibliotecaManager
        {
            get { return (Biblioteca.BibliotecaManager) DocumentManager; }
        }

        public Biblioteca Biblioteca
        {
            get { return (Biblioteca) DocumentManager.Document; }
        }

        public bool BibliotecaExists
        {
            get { return DocumentManager.DocumentExists; }
        }

        public override bool CanDoCommand(string commandName)
        {
            switch (commandName)
            {
                case "Libro":
                case "Libro.Aggiungi":
                    return BibliotecaExists;
                case "Libro.Modifica":
                    return BibliotecaExists &&
                        (Biblioteca.GetLibri().Count() > 0);
                case "Libro.ModificaCorrente":
                    return BibliotecaExists &&
                        (LibroCorrente != null);
                case "Persona":
                case "Persona.Aggiungi":
                    return BibliotecaExists;
                case "Persona.Modifica":
                    return BibliotecaExists &&
                        (Biblioteca.GetPersone().Count() > 0);
                case "Persona.ModificaCorrente":
                    return BibliotecaExists &&
                        (PersonaCorrente != null);
                case "Prestito":
                    return BibliotecaExists;
                case "Prestito.Aggiungi":
                    return BibliotecaExists &&
                        (Biblioteca.GetPersone().Count() > 0) &&
                        (Biblioteca.GetLibriDisponibili().Count() > 0);
                case "Prestito.Rimuovi":
                    return BibliotecaExists &&
                        (Biblioteca.GetPrestiti().Count() > 0);
                case "Prestito.RimuoviCorrente":
                    return BibliotecaExists &&
                        (PrestitoCorrente != null);
                default:
                    return base.CanDoCommand(commandName);
            }
        }

        public override void DoCommand(string commandName)
        {
            switch (commandName)
            {
                case "Libro.Aggiungi":
                    AggiungiLibro();
                    break;
                case "Libro.Modifica":
                    ModificaLibro();
                    break;
                case "Libro.ModificaCorrente":
                    ModificaLibro(LibroCorrente);
                    break;
                case "Persona.Aggiungi":
                    AggiungiPersona();
                    break;
                case "Persona.Modifica":
                    ModificaPersona();
                    break;
                case "Persona.ModificaCorrente":
                    ModificaPersona(PersonaCorrente);
                    break;
                case "Prestito.Aggiungi":
                    AggiungiPrestito();
                    break;
                case "Prestito.Rimuovi":
                    RimuoviPrestito();
                    break;
                case "Prestito.RimuoviCorrente":
                    RimuoviPrestito(PrestitoCorrente);
                    break;
                default:
                    base.DoCommand(commandName);
                    break;
            }
        }

        private Grid LibriGrid
        {
            get { return BibliotecaForm.LibriGrid; }
        }

        private Grid PersoneGrid
        {
            get { return BibliotecaForm.PersoneGrid; }
        }

        private Grid PrestitiGrid
        {
            get { return BibliotecaForm.PrestitiGrid; }
        }

        private Libro LibroCorrente
        {
            get
            {
                if (LibriGrid.Current == null)
                    return null;
                DataRowView dataRowView = (DataRowView) LibriGrid.Current;
                BibliotecaDataSet.LibriRow libroRow = (BibliotecaDataSet.LibriRow) dataRowView.Row;
                return new Libro(Biblioteca, libroRow);
            }
        }

        private Persona PersonaCorrente
        {
            get
            {
                if (PersoneGrid.Current == null)
                    return null;
                DataRowView dataRowView = (DataRowView) PersoneGrid.Current;
                BibliotecaDataSet.PersoneRow personaRow = (BibliotecaDataSet.PersoneRow) dataRowView.Row;
                return new Persona(Biblioteca, personaRow);
            }
        }

        private Prestito PrestitoCorrente
        {
            get
            {
                if (PrestitiGrid.Current == null)
                    return null;
                DataRowView dataRowView = (DataRowView) PrestitiGrid.Current;
                BibliotecaDataSet.PrestitiRow prestitoRow = (BibliotecaDataSet.PrestitiRow) dataRowView.Row;
                return new Prestito(Biblioteca, prestitoRow);
            }
        }

        public override void NewDocument()
        {
            base.NewDocument();
            LibriGrid.DataSource = Biblioteca.LibriTable;
            PersoneGrid.DataSource = Biblioteca.PersoneTable;
            PrestitiGrid.DataSource = Biblioteca.PrestitiTable;
        }

        public override bool OpenDocument()
        {
            bool opened = base.OpenDocument();
            if (opened)
            {
                LibriGrid.DataSource = Biblioteca.LibriTable;
                PersoneGrid.DataSource = Biblioteca.PersoneTable;
                PrestitiGrid.DataSource = Biblioteca.PrestitiTable;
            }
            return opened;
        }

        public override bool CloseDocument()
        {
            bool closed = base.CloseDocument();
            if (closed)
            {
                LibriGrid.DataSource = null;
                PersoneGrid.DataSource = null;
                PrestitiGrid.DataSource = null;
            }
            return closed;
        }

        private void AggiungiLibro()
        {
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            Libro libro = new Libro(Biblioteca);
            using (EditingForm editingForm = new EditingForm())
            {
                editingForm.Text = "Aggiungi libro";
                editingForm.SetEditableObject(libro, false);
                if (editingForm.ShowDialog() == DialogResult.OK)
                {
                    BibliotecaManager.AggiungiLibro(libro);
                }
            }
        }

        private void ModificaLibro()
        {
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            using (SelectionForm selectionForm = new SelectionForm())
            {
                selectionForm.Text = "Seleziona libro";
                selectionForm.SetSelection("Libro", "Libro da modificare:", Biblioteca.GetLibri());
                if (selectionForm.ShowDialog() == DialogResult.OK && selectionForm.GetSelectedItem() != null)
                {
                    Libro libro = (Libro) selectionForm.GetSelectedItem();
                    ModificaLibro(libro);
                }
            }
        }

        private void ModificaLibro(Libro libro)
        {
            if (libro == null)
                throw new ArgumentNullException("libro");
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            using (EditingForm editingForm = new EditingForm())
            {
                editingForm.Text = "Modifica libro";
                editingForm.SetEditableObject(libro, true);
                if (editingForm.ShowDialog() == DialogResult.OK)
                {
                    BibliotecaManager.ModificaLibro(libro);
                }
            }
        }

        private void AggiungiPersona()
        {
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            Persona persona = new Persona(Biblioteca);
            using (EditingForm editingForm = new EditingForm())
            {
                editingForm.Text = "Aggiungi persona";
                editingForm.SetEditableObject(persona, false);
                if (editingForm.ShowDialog() == DialogResult.OK)
                {
                    BibliotecaManager.AggiungiPersona(persona);
                }
            }
        }

        private void ModificaPersona()
        {
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            using (SelectionForm selectionForm = new SelectionForm())
            {
                selectionForm.Text = "Seleziona persona";
                selectionForm.SetSelection("Persona", "Persona da modificare:", Biblioteca.GetPersone());
                if (selectionForm.ShowDialog() == DialogResult.OK && selectionForm.GetSelectedItem() != null)
                {
                    Persona persona = (Persona) selectionForm.GetSelectedItem();
                    ModificaPersona(persona);
                }
            }
        }

        private void ModificaPersona(Persona persona)
        {
            if (persona == null)
                throw new ArgumentNullException("persona");
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            using (EditingForm editingForm = new EditingForm())
            {
                editingForm.Text = "Modifica persona";
                editingForm.SetEditableObject(persona, true);
                if (editingForm.ShowDialog() == DialogResult.OK)
                {
                    BibliotecaManager.ModificaPersona(persona);
                }
            }
        }

        private void AggiungiPrestito()
        {
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            using (SelectionForm selectionForm = new SelectionForm())
            {
                selectionForm.Text = "Seleziona libro e richiedente";
                selectionForm.SetSelection("Libro", "Libro richiesto:", Biblioteca.GetLibriDisponibili());
                selectionForm.AddSelection("Richiedente", "Richiedente:", Biblioteca.GetPersone());
                if (selectionForm.ShowDialog() == DialogResult.OK && selectionForm.GetSelectedItem(0) != null && selectionForm.GetSelectedItem(1) != null)
                {
                    Libro libro = (Libro) selectionForm.GetSelectedItem(0);
                    Persona persona = (Persona) selectionForm.GetSelectedItem(1);
                    Prestito prestito = new Prestito(Biblioteca, libro, persona);
                    BibliotecaManager.AggiungiPrestito(prestito);
                }
            }
        }

        private void RimuoviPrestito()
        {
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            using (SelectionForm selectionForm = new SelectionForm())
            {
                selectionForm.Text = "Seleziona prestito";
                selectionForm.SetSelection("Prestito", "Prestito da rimuovere:", Biblioteca.GetPrestiti());
                if (selectionForm.ShowDialog() == DialogResult.OK && selectionForm.GetSelectedItem() != null)
                {
                    Prestito prestito = (Prestito) selectionForm.GetSelectedItem();
                    RimuoviPrestito(prestito);
                }
            }
        }

        private void RimuoviPrestito(Prestito prestito)
        {
            if (prestito == null)
                throw new ArgumentNullException("prestito");
            if (!BibliotecaExists)
                throw new ApplicationException("!BibliotecaExists");
            BibliotecaManager.RimuoviPrestito(prestito);
        }
    }
}
